@@ -23,4 +23,4 @@
 (sb-ext:save-lisp-and-die "nodels"
  :executable t
  :toplevel #'Scat.nodels::toplevel
- :purify t)
+ :purify NIL)
