@@ -100,8 +100,8 @@
       (format t "nonexistent state~%")
       (cl-user::quit)))
   
-  (format t "Creating node: ~A mac: ~A state: ~A~%"
-          *name* *mac* *state*)
+  (format t "Creating node: ~A mac: ~A type: ~A state: ~A~%"
+          *name* *mac* *type* *state*)
 
   (set-state *name* (string-downcase *state*) t)
   (name-mac (standardize-mac *mac*) *name*)
