@@ -25,8 +25,12 @@
         #+sbcl :sb-mop
         #+lispworks :hcl
         :cl-user
-        :com.gigamonkeys.pathnames)
+        ;; :com.gigamonkeys.pathnames
+        :osicat)
   (:export 
+   node->type
+   delete-node
+   define-node
    cmd=
    expand-nodelist
    read-scat-config
@@ -79,11 +83,13 @@
   (:documentation "The error that happens when the SCAT_ROOT environment variable is unset."))
 
 (defun normalize-root (root)
+  "make sure root ends in a / character"
   (if (eql #\/ (aref root (- (length root) 1)))
       root
       (format () "~A/" root)))
 
 (defun read-scat-config (&optional config-directory)
+  "read the SCAT configuration file"
   (Scat.Cfg::read-configuration-file 
    (merge-pathnames 
     #p"etc/Scat.cfg"
@@ -136,6 +142,7 @@
   (:documentation "The error signalled when a type does not exist."))
 
 (defun ensure-type (type)
+  "ensure that the named type exists or signal an error"
   (or (type-existsp type)
       (error 'nonesistent-type :bad-type type)))
 
@@ -181,10 +188,31 @@
                (error 'nonexistent-node :node-name name))))
     (pathname-name node)))
 
+(defun name->mac (name)
+  "convert a name to a mac address"
+  (with-directory-iterator (next (merge-pathnames  Scat.Cfg::*pxe-config-dir*))
+    (loop for entry = (next)
+       while entry 
+       when (and  
+             (eq :symbolic-link (file-kind entry))
+             (equal name (namestring (read-link entry)))
+             )
+       do (return (merge-pathnames Scat.Cfg::*pxe-config-dir* entry)))))
+
+(defun delete-node (name)
+  "delete a node from the system"
+  (let ((mac (name->mac name))
+        (node-path (node-full-path name))
+        (type-path (merge-pathnames Scat.Cfg::*node-image-dir* name)))
+    (sb-posix:unlink mac)
+    (sb-posix:unlink node-path)
+    (sb-posix:unlink type-path)))
+
 (defun define-node (name mac type state)
+  "define a new node given its name, mac, type, & state"
   (set-state name state t)
-  (name-mac (standardize-mac mac))
-  (set-type name state))
+  (name-mac (standardize-mac mac) name)
+  (set-type name type))
 
 (defun set-state (name state &optional allow-nonexistent-nodes)
   "set the state of a node"
@@ -199,7 +227,7 @@
       (symlink-in-dir SCAT.CFG::*PXE-CONFIG-DIR* state node-file))))
 
 (defun set-type (name type)
-  " set the type of a node"
+  "set the type of a node"
   (multiple-value-bind (node-type-file node-type)
       (type-existsp type)
     (let ((node-file (merge-pathnames Scat.Cfg::*node-image-dir* name)))
@@ -213,7 +241,7 @@
                (merge-pathnames Scat.Cfg::*node-image-dir* name))
               (symlink-in-dir Scat.Cfg::*node-image-dir* type name))))))))
 
-(defun list-type (name)
+(defun node->type (name)
   "list the type of a node"
   (let* ((len (length (namestring Scat.Cfg::*node-image-dir*)))
          (true (namestring (truename (merge-pathnames Scat.Cfg::*node-image-dir* name))))
@@ -244,6 +272,7 @@
   (:documentation "A bad group within the mac address was found"))
 
 (defun standardize-mac (mac)
+  "standardize a MAC address"
   (let ((splitmac (or (cl-ppcre::split ":|-" (string-downcase mac))
                       (error 'unsplittable-mac-address :mac mac))))
     (when (not (equal 6 (length splitmac)))
@@ -276,6 +305,7 @@
      SCAT.CFG::*PXE-CONFIG-DIR* node mac-name)))
 
 (defun set-node-states (nodes state)
+  "set the state of a list of nodes nodes"
   (mapcar
    (lambda (node)
      (set-state node state))
@@ -304,6 +334,7 @@
   (:documentation "The error that happens when the specified MAC address has characters that are not valid ([a-f,A-F,0-9])."))
 
 (defun list-files-in-dir (directory)
+  "list the files in a directory"
   (let ((dir (sb-posix:opendir directory)))
     (loop for foo = (sb-posix:readdir dir)
           while (not (sb-alien:null-alien foo))
@@ -313,6 +344,7 @@
           collect (sb-posix:dirent-name foo))))
 
 (defun list-symlinks-in-dir (directory)
+  "return a list of all symbolic links in a directory"
   (let ((files (list-files-in-dir directory)))
     (loop as file in files
           for name =  (if (COM.GIGAMONKEYS.PATHNAMES:DIRECTORY-P file)
@@ -323,6 +355,7 @@
           collect file)))
 
 (defun list-node-files ()
+  "list the node files"
   (let ((symlinks 
          (list-symlinks-in-dir EDU.UNM.HPC.SCAT.CFG::*PXE-CONFIG-DIR*)))
     (sort
@@ -332,6 +365,7 @@
      #'string<)))
 
 (defun split-name (name)
+  "split the name into a name portion and number portion"
   (multiple-value-bind
         (matches sub-matches)
       (cl-ppcre::scan-to-strings
@@ -340,6 +374,7 @@
     sub-matches))
 
 (defun noderange (start &optional (end start))
+  "return a noderange specified by the start and end"
   (if (string-equal "ALL" start)
       (Scat.lib::list-node-files)
       (let* ((start (split-name start))
@@ -370,40 +405,12 @@
                       start-num-len
                       count)))))
 
-(defun noderange2 (start &optional (end start))
-  (if (string-equal "ALL" start)
-      (Scat.lib::list-node-files)
-      (iter (with start = (split-name start))
-            (with start-name = (aref start 0))
-            (with start-num = (aref start 1))
-            (with start-num-len = (length start-num))
-            (with end = (split-name end))
-            (with end-name = (aref end 0))
-            (with end-num = (aref end 1))
-            (with end-num-len = (length end-num))
-            (initially  ;; do error checking
-             (when (not 
-                    (eql start-num-len
-                         end-num-len))
-               (error "numeric padding issue!"))
-             (when (not 
-                    (string-equal
-                     start-name
-                     end-name))
-               (error "name mis-match!")))
-            (for count 
-                 from (read-from-string start-num)
-                 to (read-from-string end-num))
-            (collect (format () 
-                      "~A~v,'0d"
-                      start-name
-                      start-num-len
-                      count)))))
-
 (defun print-noderange (stream nodes)
+  "print a noderange to a stream"
   (format stream "~{~A~%~}" nodes))
 
 (defun expand-nodelist (nodelist)
+  "expand a nodelist"
   (if (string-equal "ALL" nodelist)
       (Scat.lib::list-node-files)
       (remove-duplicates
@@ -415,16 +422,17 @@
         (cl-ppcre::split "," nodelist))
        :test #'equal)))
 
-(defun find-dir-files (dir)
-           (let ((found ())
-                 (dir-files (list-directory dir)))
-             (mapcar
-              (lambda (file)
-                (if
-                 (not (directory-p file))
-                 (setf found (cons file found))))
-              dir-files)
-             found))
+(defun find-dir-files (directory)
+  "find the files in a directory"
+  (let ((found ())
+        (dir-files (list-directory directory)))
+    (mapcar
+     (lambda (file)
+       (if
+        (not (directory-p file))
+        (setf found (cons file found))))
+     dir-files)
+    found))
 
 (setf *macro-table* ())
 
@@ -436,6 +444,7 @@
         (format NIL "$$~A$$" thing))))
 
 (defun replace-file (orig new)
+  "expand the macros in the original file and put the results in the new file"
   (with-open-file (orig-file orig)
     (with-open-file (new-file new :direction :output :if-exists :supersede)
       (loop for line = (read-line orig-file () ())
@@ -446,6 +455,7 @@
                     "\\$\\$(.*?)\\$\\$" line #'replace-text-macros))))))
 
 (defun find-dir-dirs (dir)
+  "list all directories in a directory"
   (let ((found ())
         (dir-files (list-directory dir)))
     (mapcar
@@ -457,6 +467,7 @@
     found))
 
 (defun copy-directory-tree (orig new)
+  "copy a directory tree, expanding macros as needed"
   (UNLESS (equal orig new)
     (multiple-value-bind (exist-p device inode mode links uid gid rdev size atime mtime ctime) (sb-unix:unix-stat orig)
       (when (not (directory-p new))
@@ -492,12 +503,13 @@
 
 ;; sigint handler - cause things to exit
 (defun handle-ctrl-c (sig code scp)
+  "control-c handler: quit & die"
   (declare (ignore sig code scp))
   (cl-user::quit))
 
 ;; stuff so we can make simple comparisons of things like commands... 
-
-(defgeneric tosym (thing))
+(defgeneric tosym (thing)
+  (:documentation "convert a thing to a symbol"))
 
 (defmethod tosym ((thing string))
   (intern (string-upcase thing) :keyword))
@@ -505,7 +517,8 @@
 (defmethod tosym ((thing symbol))
   thing)
 
-(defgeneric cmd= (cmd1 cmd2))
+(defgeneric cmd= (cmd1 cmd2)
+  (:documentation "compare two commands"))
 
 (defmethod cmd= ((cmd1 symbol) (cmd2 symbol))
   (eq cmd1 cmd2))
